#!/usr/bin/env bash

script_name="$(basename $(realpath $0))"
working_dir="$(dirname $(realpath $0))"

conf_file="${working_dir}/conf/${script_name%%.*}.conf"

LOCK_DIR="/tmp/$(basename $0).$$.LOCK"
log_file="${LOCK_DIR}/${script_name%%.*}.log"

user=chris


# ----------
# Base Function(s)
# ----------

usage() {

   [[ -n  "$1" ]] && error "$1"

	[[ $terminal ]] && echo -e "
\033[1mUsage:\033[m

   $(basename $0) [OPTION] <network-name>

\033[1mDescription\033[m

   Copies gateway configuration into the home directory of the given User.
   If user is not given (at configuration file or on the command line), the
   default user '${DEFAULT_user}' is used.

   Possible values for network-name are:

                  NONE-WF
                  NONE-CKUBU
                  NONE-CKUBU2
                  NONE-CKUBU3

                  123
                  AK
                  AKB
                  ANW-URB
                  ANW-KM
                  B3-Bornim
                  BLKR
                  CKUBU
                  EBS
                  FLR-BRB
                  GA-AL
                  GA-Ersatz
                  GA-NH
                  GA-Schloss
                  JONAS
                  iRights
                  Kanzlei-Kiel
                  MBR
                  OOLM
                  OPP
                  ReachOut
                  SPR-BE
                  WF

\033[1mOptions\033[m

   -u
      The user in who's homedirectory the configueration files will be saved.

   -h
      Prints this help

   -S
      Ommit saving directory '/usr/local/src'
		

\033[1mFiles\033[m

   $conf_file: Configuration file


"

   clean_up 1


}

clean_up() {

   # Perform program exit housekeeping
   rm -rf "$LOCK_DIR"
   blank_line
   exit $1
}


echononl(){
   if $terminal ; then
      echo X\\c > /tmp/shprompt$$
      if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
         echo -e -n "   $*\\c" 1>&2
      else
         echo -e -n "   $*" 1>&2
      fi
      rm /tmp/shprompt$$
   fi
}


fatal(){
   echo ""
   if $terminal ; then
      echo -e "   [ \033[31m\033[1mFatal\033[m ] $*"
   else
      echo -e "   [ Fatal ] $*"
   fi
   echo ""
   if $terminal ; then
      echo -e "             \033[1mScript terminated\033[m.."
   else
      echo -e "             Script terminated.."
   fi
   echo ""
   rm -rf $LOCK_DIR
   exit 1
}

error (){
   echo ""
   if $terminal ; then
      echo -e "   [ \033[31m\033[1mError\033[m ] $*"
   else
      echo "   [ Error ] $*"
   fi
   echo ""
}

warn (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[33m\033[1mWarn\033[m ]  $*"
      else
         echo "   [ Warn ]  $*"
      fi
      echo ""
   fi
}

info (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[32m\033[1mInfo\033[m ]  $*"
      else
         echo "   [ Info ]  $*"
      fi
      echo ""
   fi
}

ok (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[32m\033[1mOk\033[m ]    $*"
      else
         echo "   [ Ok ]    $*"
      fi
      echo ""
   fi
}

echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33m\033[1mskipped\033[m ]"
   fi
}
trim() {
    local var="$*"
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo -n "$var"
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}



# ----------
# - Jobhandling 
# ----------

# - Run 'clean_up' for signals SIGHUP SIGINT SIGTERM
# -
trap clean_up SIGHUP SIGINT SIGTERM

# - Create lock directory '$LOCK_DIR"
#
mkdir "$LOCK_DIR"


# ----------
# - Some checks ..
# ----------

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
   terminal=true
else
   terminal=false
fi

# -Is systemd supported on this system?
# -
systemd_supported=false
systemd=$(which systemd)
systemctl=$(which systemctl)

if [[ -n "$systemd" ]] && [[ -n "$systemctl" ]] ; then
   systemd_supported=true
fi

# - Print help?
# -
if [[ "$(trim $*)" = "-h" ]] || [[ "$(trim $*)" = "--help" ]] ; then
    usage
fi

if [[ -z "$(which basename)" ]]; then
   fatal 'It seems "basename" is not installed, but needed!'
fi

if [[ -z "$(which realpath)" ]]; then
   fatal 'It seems "realpath" is not installed, but needed!'
fi



# ==========
# - Begin Main Script
# ==========

# ----------
# - Headline
# ----------

if $terminal ; then
   echo ""
   echo -e "\033[1m----------\033[m"
   echo -e "\033[32m\033[1mRunning script \033[m\033[1m$script_name\033[32m .. \033[m"
   echo -e "\033[1m----------\033[m"
   echo ""
fi


# ----------
#  Read Configurations from $conf_file
# ----------


# - Give your default values here
# -
LOGGING=false
#DEFAULT_user="chris"
DEFAULT_LOCAL_USER="chris"
DEFAULT_OMMIT_SRC_DIR=false

echononl "Read configuration file '$(basename "$conf_file")' .."
if [[ -f "$conf_file" ]]; then
   source "$conf_file"
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
   fi
else
   echo_skipped
   warn "No configuration file '$conf_file' present.\n
             Loading default values.."
fi


# ----------
# - Read commandline parameter
# ----------


while getopts hSu: opt ; do
   case $opt in
      h) usage
         ;;
      S) OMMIT_SRC_DIR=true
         ;;
      u) user="$OPTARG"
         ;;
      *) usage
         ;;
   esac
done

#[[ -z "$user" ]] && user="$DEFAULT_user"
[[ -z "$LOCAL_USER" ]] && LOCAL_USER="$DEFAULT_LOCAL_USER"
[[ -z "$OMMIT_SRC_DIR" ]] && OMMIT_SRC_DIR="$DEFAULT_OMMIT_SRC_DIR"


shift `expr $OPTIND - 1`
[ $# -ne "1" ] && usage "wrong number of arguments"


case $1 in
   CKUBU)
      network=$1
      _igmproxy=true
      _add_support_if=true
      ;;
   NONE-CKUBU|NONE-CKUBU2|NONE-CKUBU3|NONE-WF)
      network=$1
      _igmproxy=false
      _add_support_if=false
      ;;
   123|AK|AKB|ANW-URB|ANW-KM|B3-Bornim|BLKR|EBS|FLR-BRB|GA-AL|GA-Ersatz|GA-NH|GA-Schloss|iRights|JONAS|Kanzlei-Kiel|MBR|OOLM|OPP|ReachOut|SPR-BE|WF)
      network=$1
      _igmproxy=false
      _add_support_if=true
      ;;
   *)
      usage "Wrong network name"
      exit 1
esac

_realpath=`which realpath`
if [ -z "$_realpath" ]; then
   usage "realpath is not installed but needed."
fi

filesystem_prefix=""
if [ -d "/ro" ]; then
   filesystem_prefix="/ro"
fi

_network=$1
homedir_user=`eval realpath ~$LOCAL_USER`

_dir=${_network}-config


# - Delete existing Backup directory, to avoid having old (unused) files from
# - earlier configurations
# 
rm -rf ${homedir_user}/${_dir}
mkdir -p ${homedir_user}/${_dir}


# - Nameserver Bind
# -    /etc/bind/*
# -
_success=true
echononl "Save directory '${filesystem_prefix}/etc/bind' .."
if [[ -d "${filesystem_prefix}/etc/bind" ]] &&[[ "$(ls -A /etc/bind)" ]] ; then
   tar -czpf ${homedir_user}/${_dir}/bind_${_network}.tar.gz -C ${filesystem_prefix}/etc bind >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/bind_${_network}.tar.gz >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi


# - OpenVPN:
# -    /etc/openvpn/*
# -
# -    /root/.openvpn
# -
_success=true
echononl "Save directory '${filesystem_prefix}/etc/openvpn'.."
if [[ -d "${filesystem_prefix}/etc/openvpn" ]]; then
   tar -czpf ${homedir_user}/${_dir}/openvpn_${_network}.tar.gz -C ${filesystem_prefix}/etc openvpn > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/openvpn_${_network}.tar.gz >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi


_success=true
echononl "Save directory '${filesystem_prefix}/root/.openvpn' .."
if [ -d "/root/.openvpn" ]; then
   tar -czpf ${homedir_user}/${_dir}/root-dot-opennvpndir_${_network}.tar.gz -C /root .openvpn > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/root-dot-opennvpndir_${_network}.tar.gz >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

# - DynDNS
# -
_success=true
echononl "Save file '${filesystem_prefix}/etc/ddclient.conf' .."
if [ -f "${filesystem_prefix}/etc/ddclient.conf" ]; then
   cp ${filesystem_prefix}/etc/ddclient.conf ${homedir_user}/${_dir}/ddclient.conf.${_network} > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

# - DHCP Server
# - 
# -    /etc/dhcp/dhcpd.conf   -   /etc/dhcp3/dhcpd.conf 
# -    /etc/dhcp/dhcpd6.conf  -   /etc/dhcp3/dhcpd6.conf
# -
# -    /etc/dhcp/hosts.lan.conf    # dhcp failover kanzlei kiel
# -    /etc/dhcp/hosts.w-lan.conf  # dhcp failover kanzlei kiel
# -
# -    /etc/init.d/isc-dhcp6-server
# - 
_success=true
_found=false
echononl "Save DHCP configuration .."
if [ -f "${filesystem_prefix}/etc/dhcp/dhcpd.conf" ];then
   cp ${filesystem_prefix}/etc/dhcp/dhcpd.conf ${homedir_user}/${_dir}/dhcpd.conf.${_network} > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/dhcp/dhcpd6.conf" ];then
   cp ${filesystem_prefix}/etc/dhcp/dhcpd6.conf ${homedir_user}/${_dir}/dhcpd6.conf.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/dhcp3/dhcpd.conf" ];then
   cp ${filesystem_prefix}/etc/dhcp3/dhcpd.conf ${homedir_user}/${_dir}/dhcpd.conf.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/dhcp3/dhcpd6.conf" ];then
   cp ${filesystem_prefix}/etc/dhcp3/dhcpd6.conf ${homedir_user}/${_dir}/dhcpd.conf.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/init.d/isc-dhcp6-server" ]; then
   cp ${filesystem_prefix}/etc/init.d/isc-dhcp6-server ${homedir_user}/${_dir}/isc-dhcp6-server.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/dhcp/hosts.lan.conf" ]; then
   cp ${filesystem_prefix}/etc/dhcp/hosts.lan.conf ${homedir_user}/${_dir}/hosts.lan.conf.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/dhcp/hosts.w-lan.conf" ]; then
   cp ${filesystem_prefix}/etc/dhcp/hosts.w-lan.conf ${homedir_user}/${_dir}/hosts.w-lan.conf.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/default/isc-dhcp-server" ] ; then
   cp -f "${filesystem_prefix}/etc/default/isc-dhcp-server" ${homedir_user}/${_dir}/default_isc-dhcp-server.${_network} > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if $_found ; then
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

# - DSL PPP
# -
# -    /etc/ppp/peers/*
# -
# - /etc/ppp/pap-secrets
# - /etc/ppp/chap-secrets
# -
# - /etc/ppp/ip-up.d/email_notice
# -
_success=true
_found=false
echononl "Save DSL PPP configuration .."
if [[ -d "${filesystem_prefix}/etc/ppp/peers" ]] && [[ "$(ls -A /etc/ppp/peers)" ]] ; then
   _found=true
   tar -czpf ${homedir_user}/${_dir}/etc_ppp_peers_${_network}.tar.gz -C ${filesystem_prefix}/etc/ppp/ peers > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/etc_ppp_peers_${_network}.tar.gz >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
fi
if [[ -f "${filesystem_prefix}/etc/ppp/pap-secrets" ]] ; then
   _found=true
   cp ${filesystem_prefix}/etc/ppp/pap-secrets ${homedir_user}/${_dir}/pap-secrets.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
fi
if [[ -f "${filesystem_prefix}/etc/ppp/chap-secrets" ]]; then
   _found=true
   cp ${filesystem_prefix}/etc/ppp/chap-secrets ${homedir_user}/${_dir}/chap-secrets.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
fi

if [ -f "${filesystem_prefix}/etc/ppp/ip-up.d/email_notice" ];then
   _found=true
   cp ${filesystem_prefix}/etc/ppp/ip-up.d/email_notice ${homedir_user}/${_dir}/email_notice.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
fi
if $_found ; then
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

# - Postfix SMTP
# -    /etc/postfix/main.cf
# -    /etc/mailname
# -    /etc/postfix/generic
# -    /etc/postfix/generic.db
_success=true
echononl "Save Postfix SMTP configuration .."
cp ${filesystem_prefix}/etc/postfix/main.cf  ${homedir_user}/${_dir}/main.cf.${_network}
if [ -f "${filesystem_prefix}/etc/postfix/generic" ];then
   cp ${filesystem_prefix}/etc/postfix/generic ${homedir_user}/${_dir}/generic.${_network} > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   cp ${filesystem_prefix}/etc/postfix/generic.db ${homedir_user}/${_dir}/generic.db.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
fi
if [ -f "${filesystem_prefix}/etc/postfix/sasl_passwd" ]; then
   cp ${filesystem_prefix}/etc/postfix/sasl_passwd ${homedir_user}/${_dir}/sasl_passwd.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   cp ${filesystem_prefix}/etc/postfix/sasl_passwd.db ${homedir_user}/${_dir}/sasl_passwd.db.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
fi
cp ${filesystem_prefix}/etc/mailname ${homedir_user}/${_dir}/mailname.${_network} >> $log_file 2>&1
[[ $? -ne 0 ]] && _success=false
if $_success ; then
   echo_ok
else
   echo_failed
   error "$(cat "$log_file")"
fi

# - Netzwerk:
# -    /etc/network/interfaces
# -    /etc/resolv.conf
# -    /etc/hostname
# -    /etc/hosts
_success=true
echononl "Save Network configuration .."
cp ${filesystem_prefix}/etc/hostname ${homedir_user}/${_dir}/hostname.${_network} > $log_file 2>&1
[[ $? -ne 0 ]] && _success=false
cp ${filesystem_prefix}/etc/hosts ${homedir_user}/${_dir}/hosts.${_network} >> $log_file 2>&1
[[ $? -ne 0 ]] && _success=false
cp ${filesystem_prefix}/etc/network/interfaces  ${homedir_user}/${_dir}/interfaces.${_network} >> $log_file 2>&1
[[ $? -ne 0 ]] && _success=false
cp ${filesystem_prefix}/etc/resolv.conf ${homedir_user}/${_dir}/resolv.conf.${_network} >> $log_file 2>&1
[[ $? -ne 0 ]] && _success=false
if $_success ; then
   echo_ok
else
   echo_failed
   error "$(cat "$log_file")"
fi


# - /root/bin directory
# -
_success=true
echononl "Save directory '${filesystem_prefix}/root/bin' .."
if [[ -d "${filesystem_prefix}/root/bin" ]] && [[ "$(ls -A /root/bin)" ]] ; then
   tar -czpf ${homedir_user}/${_dir}/root_bin_${_network}.tar.gz -C ${filesystem_prefix}/root bin > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/root_bin_${_network}.tar.gz >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

# - /usr/local/src directory
# -
_success=true
echononl "Save directory '${filesystem_prefix}/usr/local/src' .."
if $OMMIT_SRC_DIR ; then
   echo_skipped
else
   if [[ -d "${filesystem_prefix}/usr/local/src" ]] && [[ "$(ls -A /usr/local/src)" ]] ; then
      tar -czpf ${homedir_user}/${_dir}/usr_local_src_${_network}.tar.gz -C ${filesystem_prefix}/usr/local src > $log_file 2>&1
      [[ $? -ne 0 ]] && _success=false
      chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/usr_local_src_${_network}.tar.gz >> $log_file 2>&1
      [[ $? -ne 0 ]] && _success=false
      if $_success ; then
         echo_ok
      else
         echo_failed
         error "$(cat "$log_file")"
      fi
   else
      echo_skipped
   fi
fi

# - /usr/local/sbin directory
# -
_success=true
echononl "Save directory '${filesystem_prefix}/usr/local/sbin' .."
if [[ -d "${filesystem_prefix}/usr/local/sbin" ]] && [[ "$(ls -A /usr/local/sbin)" ]] ; then
   tar -czpf ${homedir_user}/${_dir}/usr_local_sbin_${_network}.tar.gz -C ${filesystem_prefix}/usr/local sbin > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/usr_local_sbin_${_network}.tar.gz >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

# - Firewall
# -
# -    /etc/systemd/system/ipt-firewall.service
# -    /etc/systemd/system/ip6t-firewall.service
# -
# -    /etc/init.d/ipt-firewall
# -    /etc/init.d/ipt-firewall
# -
# -    /etc/ipt-firewall/*
# -
_success=true
_found=false
echononl "Save Firewall configuration .."
if [[ -d "${filesystem_prefix}/etc/ipt-firewall" ]] &&  [[ "$(ls -A /etc/ipt-firewall)" ]] ; then
   tar -czpf ${homedir_user}/${_dir}/etc_ipt-firewall_${_network}.tar.gz -C ${filesystem_prefix}/etc ipt-firewall > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/etc_ipt-firewall_${_network}.tar.gz >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi

if [[ -f "${filesystem_prefix}/etc/systemd/system/ipt-firewall.service" ]]; then
   cp ${filesystem_prefix}/etc/systemd/system/ipt-firewall.service ${homedir_user}/${_dir}/ipt-firewall.service.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [[ -f "${filesystem_prefix}/etc/systemd/system/ip6t-firewall.service" ]]; then
   cp ${filesystem_prefix}/etc/systemd/system/ip6t-firewall.service ${homedir_user}/${_dir}/ip6t-firewall.service.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi

if  [ -f "${filesystem_prefix}/etc/init.d/ipt-firewall" ]; then
   cp ${filesystem_prefix}/etc/init.d/ipt-firewall ${homedir_user}/${_dir}/ipt-firewall.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/init.d/ip6t-firewall" ]; then
   cp ${filesystem_prefix}/etc/init.d/ip6t-firewall ${homedir_user}/${_dir}/ip6t-firewall.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi

if  [ -f "${filesystem_prefix}/etc/init.d/ipt-firewall-gateway" ]; then
   cp ${filesystem_prefix}/etc/init.d/ipt-firewall-gateway ${homedir_user}/${_dir}/ipt-firewall-gateway.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [ -f "${filesystem_prefix}/etc/init.d/ip6t-firewall-gateway" ]; then
   cp ${filesystem_prefix}/etc/init.d/ip6t-firewall-gateway ${homedir_user}/${_dir}/ip6t-firewall-gateway.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if $_found ; then
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

# - Check Script Onlinestatus
# -
# -    /etc/systemd/system/check_net.service
# -
# -    /usr/local/sbin/check_net.sh
# -    /usr/local/sbin/netconfig.sh
# -
# -    /etc/check_net/*
# -
# -    /etc/logrotate.d/check_net
# -
_success=true
_found=false
echononl "Save Configuration of script 'check_net' .."
if [[ -d "${filesystem_prefix}/etc/check_net" ]] && [[ "$(ls -A /etc/check_net)" ]] ; then
   tar -czpf ${homedir_user}/${_dir}/etc_check_net_${_network}.tar.gz -C ${filesystem_prefix}/etc check_net > $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   chown ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir}/etc_check_net_${_network}.tar.gz >> $log_file 2>&1
   _found=true
fi
if [[ -f "${filesystem_prefix}/etc/systemd/system/check_net.service" ]] ; then
   cp ${filesystem_prefix}/etc/systemd/system/check_net.service ${homedir_user}/${_dir}/check_net.service.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if [[ -f "${filesystem_prefix}/etc/logrotate.d/check_net" ]] ; then
   cp ${filesystem_prefix}/etc/logrotate.d/check_net ${homedir_user}/${_dir}/check_net-logrotate.${_network} >> $log_file 2>&1
   [[ $? -ne 0 ]] && _success=false
   _found=true
fi
if $_found ; then
   if $_success ; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

# - Other files
# -
# -    /etc/rc.local
# -    /var/spool/cron/crontabs/root
# -    /usr/local/igmpproxy/etc/igmpproxy.conf
# -    /etc/aiccu.conf
# -    /etc/default/aiccu
# -    /etc/radvd.conf
# -    /etc/hostapd/hostapd.conf
# -
_success=true
echononl "Save file '${filesystem_prefix}/etc/rc.local' .."
if [ -f "${filesystem_prefix}/etc/rc.local" ]; then 
   cp ${filesystem_prefix}/etc/rc.local ${homedir_user}/${_dir}/rc.local.${_network} > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

echononl "Save file '${filesystem_prefix}/var/spool/cron/crontabs/root' .."
if [ -f "${filesystem_prefix}/var/spool/cron/crontabs/root" ]; then
   cp ${filesystem_prefix}/var/spool/cron/crontabs/root ${homedir_user}/${_dir}/cron_root.${_network} > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

echononl "Save file '${filesystem_prefix}/usr/local/igmpproxy/etc/igmpproxy.conf' .."
if [ -f "${filesystem_prefix}/usr/local/igmpproxy/etc/igmpproxy.conf" ]; then
   cp ${filesystem_prefix}/usr/local/igmpproxy/etc/igmpproxy.conf ${homedir_user}/${_dir}/igmpproxy.conf.${_network} > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

echononl "Save file '${filesystem_prefix}/etc/aiccu.conf' .."
if [ -f "${filesystem_prefix}/etc/aiccu.conf" ]; then
   cp ${filesystem_prefix}/etc/aiccu.conf ${homedir_user}/${_dir}/aiccu.conf.${_network} > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

echononl "Save file '${filesystem_prefix}/etc/default/aiccu' .."
if [ -f "${filesystem_prefix}/etc/default/aiccu" ]; then
   cp ${filesystem_prefix}/etc/default/aiccu ${homedir_user}/${_dir}/aiccu.${_network} > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

echononl "Save file '${filesystem_prefix}/etc/radvd.conf' .."
if [ -f "${filesystem_prefix}/etc/radvd.conf"  ]; then
   cp ${filesystem_prefix}/etc/radvd.conf ${homedir_user}/${_dir}/radvd.conf.${_network} > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

echononl "Save file '${filesystem_prefix}/etc/hostapd/hostapd.conf' .."
if [ -f "${filesystem_prefix}/etc/hostapd/hostapd.conf"  ]; then
   cp ${filesystem_prefix}/etc/hostapd/hostapd.conf ${homedir_user}/${_dir}/hostapd.conf.${_network} > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
else
   echo_skipped
fi

blank_line

echononl "Change Ownership of directory '${homedir_user}/${_dir}' .. "
chown -R ${LOCAL_USER}:${LOCAL_USER} ${homedir_user}/${_dir} > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   error "$(cat "$log_file")"
fi

blank_line
clean_up 0
